#  R-Version:    R 2.12.1  (2010-12-16)

#  Required input files:
#
#   TNBC_Metag_ProbesLists.txt 
#   GSE31519_complete_dataset.txt 
#   GSE31519_TNBC_SampleInfo_BCR.txt 
#   
# How to obtain:
#
# The file TNBC_Metag_ProbesLists.txt is also included in the "Additional data file" zip package
#
# GSE31519_TNBC_SampleInfo_BCR.txt  is available from
# ftp://ftp.ncbi.nih.gov/pub/geo/DATA/supplementary/series/GSE31519/GSE31519%5FTNBC%5FSampleInfo%5FBCR%2Etxt%2Egz
#
# GSE31519_complete_dataset.txt  is available from
# ftp://ftp.ncbi.nih.gov/pub/geo/DATA/supplementary/series/GSE31519/GSE31519%5Fcomplete%5Fdataset%2Etxt%2Egz
#
# This last file from the GEO database has a rather complicated structure with a gap of 23 columns 
#   since it also contains samples from the ArrayExpress database which they have moved to the end.
#   The file is used to rebuild the complete dataset of 579 samples here:
#   

samples <- read.delim("GSE31519_complete_dataset.txt", colClasses= "character", skip=1, nrows=1, header=FALSE)   # read the sample names from row-2
data <- read.delim("GSE31519_complete_dataset.txt", skip=4,header=FALSE)  # read data starting row-5
colnames(data)=t(samples)       # add sample names from row-2 as column names
data=data[,!samples==""]        # remove blank gap-columns with no sample-title in row-2 from expr data
rm(samples)   # delete used variable
rownames(data)=data[,1]     # use first column with Affy IDs as rownames
cd_compl_579=data[,-1]    # remove first column with Affy IDs
rm(data)  # delete used variable
cd_compl_579=cd_compl_579[,sort(colnames(cd_compl_579))]   # sort columns by sample name


header_579 <- read.delim("GSE31519_TNBC_SampleInfo_BCR.txt",colClasses = "character", row.names="SampleNames",h=T)   # all sample information

#  Select the n=394 comparable TNBC samples from n=579 TNBC Affymetrix MAS5 data

#  Variables used:
#  cd_compl_579 ==> chip-data complete (data.frame, 579 samples in columns, 22283 probesets in rows)
#  header_579  ==> sample infos including dataset allocation (data.frame, 579 samples in columns)
#  n_probes ==> number of ProbeSets (rows in cd_compl_579)
#  n_col_cd ==> number of columns/samples in cd_compl_579
#  tcdm_579  ==> transposed chip data matrix
#  t_header_579 ==> transposed sample infos (data.frame)
#  ds_579 ==> dataset allocation of 579 samples from header_579
#  tds_579  ==>  transposed ds_579 (dataset allocation as matrix)
#  tdsn_579  ==> as.numeric(tds_579)  (vector)
#

n_col_cd=ncol(cd_compl_579)
n_probes=nrow(cd_compl_579)
ds_579=header_579["datas_new09",]    # dataset allocation of 579 samples from header_579
tds_579=t(ds_579)  # transposed ds_579 (dataset allocation as matrix)
tdsn_579=as.numeric(tds_579)   # transposed dataset vector
tcdm_579=t(cd_compl_579)   # transposed chip data matrix
t_header_579=as.data.frame(t(header_579))  # transposed sample info header as data.frame
ds_mean=by(tcdm_579, tdsn_579, mean)  # means within individual datasets, 
                                     #  List is sorted by numeric dataset tdsn_579
tcdm_mean=apply(tcdm_579,2,mean)
        #   generates named list of 22283 means, which can be indexed by tcdm_mean[probes]
tcdm_stdev=apply(tcdm_579,2, sd)   # the same for the StdDev


# Calculate comparability metrics for the datasets:
# (calculate sum of squared differences of dataset-mean from total-mean for all probesets)
# Define variables
n_datas=length(ds_mean)   # number of datasets
diff_to_mean= matrix(0,nrow=n_probes,ncol=n_datas) # matrix of differences from mean
nrm_diff_to_mean= matrix(0,nrow=n_probes,ncol=n_datas) # matrix of NORMALIZED diff from mean

for (probes in 1:n_probes)   # loop for all probesets
{

for (i in 1:n_datas)
# calculate for each dataset diff from global-mean of all datasets
# and save in matrix "diff_to_mean"
{
diff_to_mean[probes,i]=ds_mean[[i]][probes]- tcdm_mean[probes]
# again the same but normalize by dividing through StdDev
nrm_diff_to_mean[probes,i]=(ds_mean[[i]][probes]- tcdm_mean[probes]) / tcdm_stdev[probes]
}
}
# calculate squares of differences
squ_diff=diff_to_mean^2
squ_nrm_diff= nrm_diff_to_mean^2
# sum of squared differences by column
sum_squ_diff=apply(na.omit(squ_diff),2,sum)
sum_squ_nrm_diff=apply(na.omit(squ_nrm_diff),2,sum)
#  important: these vectors are still sorted by numeric dataset tdsn_579 !

# summarize results
comparab=data.frame(sort(unique(tdsn_579)),sum_squ_diff,sum_squ_nrm_diff)
names(comparab)=c("dataset","sum_squ_diff","sum_squ_nrm_diff")
sort.comparab=comparab[order(comparab$sum_squ_nrm_diff),]

# integrate normalized comparab data in sample info in t_header_579
for (i in 1:n_col_cd) 
    {t_header_579$comparab_nrm[i]= comparab$sum_squ_nrm_diff[comparab$dataset==tdsn_579[i]]}
# remove temporary variables:
rm(diff_to_mean, nrm_diff_to_mean, ds_mean,tcdm_mean, tcdm_stdev, squ_diff, squ_nrm_diff, sum_squ_diff, sum_squ_nrm_diff)

plot(sort(t_header_579$comparab_nrm),type="l")
abline(8000,0,col="red")



###################################################################################################




#  Select a subset of datasets with lowest comparability metric
# Select a subset of comparab by defining criteria:
compar_subset= subset(comparab, subset= sum_squ_nrm_diff < 8000)
# vector of corresponding datasets:
datas_subset=compar_subset$dataset
# generate logical vector FALSE/TRUE for the complete dataset of 579 TNBC:
subset_index_vector=(tdsn_579 %in% datas_subset)
# query selected samples from transposed chipdata matrix:
tcdm_394=tcdm_579[subset_index_vector , ]
# query  corresponding transposed dataset vector:
tdsn_394=tdsn_579[subset_index_vector]
# query selcted samples from NOT-transposed dataset (and corresponding header and t_header):
cd_compl_394= cd_compl_579[,subset_index_vector]
header_394= header_579[,subset_index_vector]
t_header_394= t_header_579[subset_index_vector,]
  


# Calculate Kruskal-Wallis statistics for all probesets according to their association with the dataset vector among the 394 TNBC samples:

# Variables used:    tcdm_394, tdsn_394, n_probes, cd_compl_394
# Newly generated variables:    kruskal_param

kruskal_result=kruskal.test(tcdm_394[,1],tdsn_394)  # Define variable as "list" by performing test once
kruskal_stat= matrix(0,nrow=n_probes)  # matrix for chi-statistics
kruskal_p=matrix(0,nrow=n_probes)  # matrix for p-values

# loop perfoming kruskal-test for each probeset (chip-data vs. dataset-vector)
# the results of each test are variables of type LIST, which are combined by indexing with [[i]]
for (i in 1:n_probes)
{
kruskal_result[[i]]=kruskal.test(tcdm_394[,i], tdsn_394)   # results as LIST
kruskal_stat[i]=kruskal_result[[i]]$statistic  # read statistics from LIST in matrix
kruskal_p[i]=kruskal_result[[i]]$p.value   # read p-values from LIST in matrix
}
# summarize statistics and p-values and probeset names in one dataframe:
kruskal_param= data.frame(row.names=rownames(cd_compl_394),kruskal_stat,kruskal_p)  # dataframe with results
# remove temporary variables:
rm(kruskal_result, kruskal_stat, kruskal_p)
  
hist(kruskal_param$kruskal_stat,breaks=30,col="dark blue") # histogram for all probesets



###################################################################################################

#  Input-file required:   TNBC_Metag_ProbesLists.txt

# Calculation of metagene-values for all 394 samples:
# Variables:
# metagenes_probelists ==>  lists of probesets representing the different metagenes
# n_metagenes ==> number of different metagenes
# metag_means ==> matrix for metagene means of all samples
# cd_probes_c ==> probeset-names as character from  rownames of cd_compl_394

# Variable used from above:    cd_compl_394

# load probelists for metagenes and respective header-line

metagenes_probelists <- read.delim("TNBC_Metag_ProbesLists.txt",h=T)
n_metagenes= ncol(metagenes_probelists)
metag_means=matrix(0,ncol= ncol(cd_compl_394),nrow=n_metagenes) # define matrix for results
cd_probes_c =rownames(cd_compl_394)  # probeset-names as character

for (i in 1: n_metagenes)   # loop for metagen calculation
{
probes_of_metag=as.character(metagenes_probelists[,i])  # retrieve probes as character
# generate logical vector of selected probesets 
metag_subset_index_vector=(cd_probes_c %in% probes_of_metag) 
# calculate metagene-mean using logical vector (for all samples)
metag_mean_TF=by(as.matrix(cd_compl_394),metag_subset_index_vector,mean) 
    # provides mean values for TRUE (metagene probesets) and FALSE (not used)
metag_means[i,]= metag_mean_TF$'TRUE'
}
# Summarize results:
rownames(metag_means)=colnames(metagenes_probelists)
colnames(metag_means)=colnames(cd_compl_394)
metag_df=as.data.frame(t(metag_means))  # transposed as dataframe (to query metagenes by name)
# remove temporary variables:
rm(n_metagenes, metag_means, cd_probes_c, probes_of_metag, metag_subset_index_vector, metag_mean_TF)

# Kruskal-Wallis-stats of metagene-probesets:

metagene_kruskal_stats=vector(mode="list",length=ncol(metagenes_probelists))
names(metagene_kruskal_stats)=colnames(metagenes_probelists)
median_metagene_kruskal_stats=vector(mode = "numeric", length = ncol(metagenes_probelists))
names(median_metagene_kruskal_stats)=colnames(metagenes_probelists)

for (i in 1: ncol(metagenes_probelists))   # loop for kruskal-statistic of metagenes calculation
{
probes_of_metag=as.character(metagenes_probelists[,i])  # retrieve probes as character
metagene_kruskal_stats[[i]]=kruskal_param$kruskal_stat[rownames(kruskal_param[,]) %in% probes_of_metag]
median_metagene_kruskal_stats[i]=median(metagene_kruskal_stats[[i]])
}

metagene_kruskal =cbind(metagene_kruskal_stats,median_metagene_kruskal_stats)[order(median_metagene_kruskal_stats),]
# remove temporary variables:
rm(metagene_kruskal_stats, median_metagene_kruskal_stats, probes_of_metag, i)
boxplot(metagene_kruskal[,1])


